#!/usr/bin/python

########################################################################
# get-cert.py
# (c) by Michael Stroeder, michael.stroeder@inka.de
########################################################################

Version='0.5.3'

########################################################################
#
# CGI-BIN for downloading certificates and CRLs
#
# Input:
#
# PATH_INFO
# - Name of CA in openssl.cnf (section [ca] of openssl.cnf)
# - Type of certificate ('email', 'user', 'ca', 'crl')
# QUERY_STRING only for ('email', 'user')
# - Serial number of desired certificate
#   max. 8 digits hexadecimal (32 Bit)
#
# Examples:
#   get-cert.py/Persona/ca
#   sends CA certificate of CA "Persona"
#
#   get-cert.py/Persona/email?01
#   sends client certificate of CA "Persona" with serial 0x01
#
#   get-cert.py/Server/crl
#   sends CRL of CA "Server"
#
# Output:
#
#   Content-type: application/x-x509-user-cert or
#   Content-type: application/x-x509-email-cert or
#   Content-type: application/x-x509-ca-cert or
#   Content-type: application/x-pkcs7-crl
#   and the desired certificate
#   (output format DER: 'openssl [x509|crl] -outform der')
#
########################################################################

import sys, os, string, regex, pycacnf, htmlbase, openssl

from pycacnf import opensslcnf, pyca_section

# Wir lesen rein gar nix von Standardeingabe => gleich dicht machen
sys.stdin.close()

# Path to openssl executable
OpenSSLExec = pyca_section.get('OpenSSLExec','/usr/local/ssl/bin/openssl')
caCertFormat = string.lower(pyca_section.get('caCertFormat','der'))

# Ein paar Umgebungsvariablen auslesen, welche der Apache liefert
request_method  = os.environ.get('REQUEST_METHOD','')
query_string    = os.environ.get('QUERY_STRING','')
path_info       = os.environ.get('PATH_INFO','')[1:]

# Hier die ueblichen Paranoid-Pruefungen der Parameter

if request_method!='GET':
  # Skript nicht mit GET aufgerufen
  htmlbase.PrintErrorMsg('Wrong method.')
  sys.exit(0)

# Bezeichnung der Sub-CA und MIME-Typ aus PATH_INFO splitten
try:
  ca_name, certtype = string.split(path_info,'/',1)
  certtype=string.lower(certtype)
except ValueError:
  htmlbase.PrintErrorMsg('Invalid parameter format.')
  sys.exit(0)

# Name der CA prfen
if not opensslcnf.data['ca'].has_key(ca_name):
  # CA-Definition nicht in openssl-Konfiguration enthalten
  htmlbase.PrintErrorMsg('Unknown certificate authority "%s".' % ca_name)
  sys.exit(0)

# Zertifikattyp prfen
if not (certtype in ['email','user','ca','crl']):
  # Zertifikattyp war nicht gueltig
  htmlbase.PrintErrorMsg('Invalid certificate type "%s"' % certtype)
  sys.exit(0)

if (not certtype in ['ca','crl']) and (not query_string):
  # keine Seriennummer
  htmlbase.PrintErrorMsg('No serial number.')
  sys.exit(0)

if len(query_string)>8:
  # Seriennummer mit mehr 32 Bit
  htmlbase.PrintErrorMsg('Serial number too long.')
  sys.exit(0)

if regex.match('[0-9a-fA-F]*',query_string) != len(query_string):
  # Parameter war keine Hex-Nummer
  htmlbase.PrintErrorMsg('Serial number not in hexadecimal format.')
  sys.exit(0)

# Hier Anfrage bearbeiten

ca_section=opensslcnf.data[opensslcnf.data['ca'][ca_name]]
ca_dir = ca_section.get('dir','')

if certtype=='ca':

  # Abruf des CA-Zertifikates
  ca_certificate = ca_section.get('certificate','$dir/cacert.pem')
  certfilename = string.replace(ca_certificate,'$dir',ca_dir)
  opensslcommand = '%s x509 -inform "%s" -outform der -in "%s"' %(OpenSSLExec,caCertFormat,certfilename)
  mimetype = 'application/x-x509-ca-cert'

elif certtype=='crl':

  ca_crl = ca_section.get('crl','$dir/crl.pem')
  certfilename = string.replace(ca_crl,'$dir',ca_dir)
  opensslcommand = '%s crl -inform "%s" -outform der -in "%s"' %(OpenSSLExec,caCertFormat,certfilename)
  mimetype = 'application/x-pkcs7-crl'

else:

  ca_database = string.replace(ca_section.get('database','$dir/index.txt'),'$dir',ca_dir)
  ca_cert_dir = string.replace(ca_section.get('certs','$dir/certs'),'$dir',ca_dir)

  # Abruf eines Zertifikates mittels Seriennummer
  serialnumber=string.atoi(query_string,16)

  entry = openssl.db.GetEntrybySerial(ca_database,serialnumber)

  # Kein entsprechender Eintrag gefunden
  if not entry:
    htmlbase.PrintErrorMsg('Certificate not found.')
    sys.exit(0)

  # Zertifikat ist ungueltig
  if entry[openssl.db.DB_type]!=openssl.db.DB_TYPE_VAL:
    htmlbase.PrintErrorMsg('Certificate invalid.')
    sys.exit(0)

  certfilename = os.path.join(ca_cert_dir,entry[openssl.db.DB_serial]+'.pem')
  opensslcommand = '%s x509 -inform "%s" -outform der -in "%s"' %(OpenSSLExec,caCertFormat,certfilename)
  mimetype = 'application/x-x509-%s-cert' % (certtype)

if caCertFormat=='der':

  # Direkt das DER-kodierte Zertifikat laden
  certfilename = '%s.der' % os.path.splitext(certfilename)[0]

  # Existiert die Datei?
  if not os.path.isfile(certfilename):
    htmlbase.PrintErrorMsg('Certificate file not found.')
    sys.exit(0)

  certfile = open(certfilename,'r')

else:

  # Existiert die Datei?
  if not os.path.isfile(certfilename):
    htmlbase.PrintErrorMsg('Certificate file not found.')
    sys.exit(0)

  # openssl die DER-Datei auf stdout schreiben lassen
  certfile = os.popen(opensslcommand)

# Zertifikat aus Dateiobject certfile lesen
cert=''
buf = certfile.read()
while buf:
  cert = cert + buf
  buf = certfile.read()
rc = certfile.close()

if rc:
  htmlbase.PrintErrorMsg('Error %s reading certificate.' % (rc))
  sys.exit(0)

# Hier nun letztenendes wirklich MIME-Type und Zertifikat auf stdout schreiben
sys.stdout.flush()
sys.stdout.write('Content-type: %s\n\n' % mimetype)
sys.stdout.write(cert)

sys.exit(0)
