##############################################################################
# cgiforms.py Version 0.6.3
# (c) by Michael Stroeder, michael.stroeder@propack-data.de
##############################################################################
# Klassen zum Behandeln <FORM>s und zu CGI-BINs bermittelten Forminhalten
#
# Anmerkungen:
# - Eingabefelder mit gleichen name-Attributen werden logisch zusammengefasst
#   und muessen in der selben Reihenfolge registriert und angegeben werden.
# - Es werden nur Parameter aus dem QUERY_STRING (GET) oder einzelne
#   Datensaetze vom Typ application/x-www-form-urlencoded (POST) als
#   Parameter angenommen.
###############################################################################

import sys, os, string, re, urllib, charset

###############################################################################
# Klassen zur Verwaltung von einzelnen Form-Feldern
###############################################################################

###############################################################################
# Basisklasse fuer beliebige Eingabe-Felder.
# Sollte nicht direkt benutzt werden,
# spezielle Felder sollten von dieser Klasse abgeleitet werden 
###############################################################################

class formFieldClass:

  def __init__(self,name='',text='',maxlength=0,pattern='',default='',required=0):
    self.name      = name
    self.text      = text
    self.maxlength = maxlength
    self.regex     = re.compile(pattern)
    self.default   = default
    self.required  = required
    self.content   = ''
    self.counter   = 0

  # Feldinhalt setzen
  def put(self,input):
    self.content = input

  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self):
    pass

  # Simple Text-Ausgabe des Feldinhalts
  def textprint(self):
    return self.content
    
  # HTML-Ausgabe des Feldinhalts
  def contentprint(self):
    return charset.iso2html4(self.content)

###############################################################################
# Klasse fuer <TEXTAREA>
###############################################################################

class formTextareaClass(formFieldClass):

  def __init__(self,name='',text='',maxlength=0,pattern='',default='',required=0,rows=0,cols=0):
    formFieldClass.__init__(self,name,text,maxlength,pattern,default,required)
    self.rows = rows
    self.cols = cols

  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self):
    return '<TEXTAREA NAME="%s" ROWS=%d COLS=%d>%s</TEXTAREA>' % \
           (self.name,self.rows,self.cols,self.default)

###############################################################################
# Klasse fuer <INPUT>
###############################################################################

class formInputClass(formFieldClass):

  def __init__(self,name='',text='',maxlength=0,pattern='',default='',required=0,size=0):
    formFieldClass.__init__(self,name,text,maxlength,pattern,default,required)
    if size:
      self.size = size
    else:
      self.size = maxlength
  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self):
    return '<INPUT NAME="%s" MAXLENGTH=%d SIZE=%d VALUE="%s">' % \
           (self.name,self.maxlength,self.size,self.default)


###############################################################################
# Klasse fuer <INPUT TYPE=HIDDEN>
###############################################################################

class formHiddenInputClass(formInputClass):

  def __init__(self,name='',text='',maxlength=0,pattern='',default='',required=0,show=0):
    formFieldClass.__init__(self,name,text,maxlength,pattern,default,required)
    self.show = show

  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self):
    if self.show:
      return '<INPUT TYPE=HIDDEN NAME="%s" VALUE="%s">%s' % \
             (self.name,self.default,self.default)
    else:    
      return '<INPUT TYPE=HIDDEN NAME="%s" VALUE="%s">' % \
             (self.name,self.default)


###############################################################################
# Klasse fuer <INPUT TYPE=password>
# ist von <INPUT> getrennte Klasse, da eigene contentprint-Methode
###############################################################################

class formPasswordClass(formFieldClass):

  def __init__(self,name='',text='',maxlength=0,pattern='',required=0,size=0):
    formFieldClass.__init__(self,name,text,maxlength,pattern,'',required)
    if size:
      self.size = size
    else:
      self.size = maxlength
  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self):
    return '<INPUT NAME="%s" MAXLENGTH=%d SIZE=%d TYPE=password>' % \
           (self.name,self.maxlength,self.size)
  # Simple Text-Ausgabe des Feldinhalts
  def textprint(self):
    return len(self.content)*'*'
      # HTML-Ausgabe des Feldinhalts
  def contentprint(self):
    return len(self.content)*'*'


###############################################################################
# Klasse fuer <SELECT>
###############################################################################

class formSelectClass(formFieldClass):

  def __init__(self,name='',text='',options=[],default='',required=0,size=1,multiple=0):
    # pattern und maxlength werden aus den Optionen zusammengebaut
    if len(options):
      pattern   = '(%s)' % string.join(options,'|')
      maxlength = len(options[0])
      for i in options[1:]:
        if len(i)>maxlength:
	  maxlength=len(i)
    else:
      pattern   = ''
      maxlength = 0
    formFieldClass.__init__(self,name,text,maxlength,pattern,default,required)
    self.options  = options
    self.size     = size
    self.multiple = multiple

  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self):
    s = ['<SELECT NAME="%s" SIZE=%d %s>' % (self.name,self.size," MULTIPLE"*self.multiple)]
    for i in self.options:
      s.append('<OPTION%s>%s' % (' SELECTED'*(i==self.default),i))
    s.append('</SELECT>')
    return string.join(s,'')

###############################################################################
# Klasse fuer <INPUT TYPE=RADIO>
###############################################################################

class formRadioClass(formFieldClass):

  def __init__(self,name='',text='',options='',default='',required=0):
    # pattern und maxlength werden aus den Optionen zusammengebaut
    if len(options):
      pattern   = '(%s)' % string.join(options,'|')
      maxlength = len(options[0])
      for i in options[1:]:
        if len(i)>maxlength:
	  maxlength=len(i)
    else:
      pattern   = ''
      maxlength = 0
    formFieldClass.__init__(self,name,text,maxlength,pattern,default,required)
    self.options  = options

  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self):
    for i in self.options:
      return '<INPUT TYPE=radio NAME="%s" VALUE="%s"%s>%s<BR>' % \
             (self.name,i,' CHECKED'*(i==self.default),i)

###############################################################################
# Klasse fuer <INPUT TYPE=CHECKBOX>
###############################################################################

class formCheckboxClass(formFieldClass):

  def __init__(self,name='',text='',value='',checked=0,required=0):
    pattern      = value
    maxlength    = len(value)
    formFieldClass.__init__(self,name,text,maxlength,pattern,'',required)
    self.value   = value
    self.checked = checked

  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self):
    return '<INPUT TYPE=checkbox NAME="%s" VALUE="%s"%s>' % \
           (self.name,self.value,' CHECKED'*self.checked)

###############################################################################
# Klasse fuer <KEYGEN>
###############################################################################

class formKeygenClass(formFieldClass):

  def __init__(self,name='',text='',maxlength=0,required=0):
    pattern      = r'[ -z\n]*'
    formFieldClass.__init__(self,name,text,maxlength,pattern,required)

  # Feldinhalt setzen
  def put(self,input):
    self.content = string.translate(input, string.maketrans("",""),"\n")

  # HTML-Ausgabe des Eingabefeldes
  def inputfield(self,challenge):
    return '<KEYGEN NAME="%s" CHALLENGE="%s">' % \
           (self.name,challenge)

  # HTML-Ausgabe des Feldinhalts
  def contentprint(self):
    return '%d Bytes' % (len(self.content))

#############################################################
# Klassen zur Verwaltung einer Ansammlung von Eingabefeldern
#############################################################

# Klassen zur Fehlerbehandlung von formClass
class formException(Exception):
  pass

# Gesamtlaenge der eingegebenen Daten zu lang
class formContentLengthException(formException):
  def __init__(self,contentlength):
    self.contentlength = contentlength
      
# Parameter mit unbekanntem Namen      
class formParamNameException(formException):
  def __init__(self,name):
    self.name = name

# Parameter mit unbekanntem Namen      
class formParamNameMissing(formException):
  def __init__(self,name,text):
    self.name     = name
    self.text     = text

# Inhalt eines Parameters ist ungueltig      
class formParamContentException(formException):
  def __init__(self,name,text,content,reqregex):
    self.name     = name
    self.text     = text
    self.content  = content
    self.reqregex = reqregex

# Zuviele Parameter mit selbem Namen      
class formParamStructException(formException):
  def __init__(self,name,count):
    self.name  = name
    self.count = count

# Angegebener Parameter ist zu lang
class formParamLengthException(formException):
  def __init__(self,name,text,length):
    self.name   = name
    self.text   = text
    self.length = length

#############################################################
# Die eigentlichen Klasse fuer ein <FORM>
#############################################################
      
class formClass:

  def __init__(self):
    # Dictionary der Eingabefelder-Objekte
    # {name:[FormFieldClass]}
    self.field            = {}
    # Dictionary mit Ordnungszaehler der Eingabefelder-Objekte
    # {name:len(field)}
    self.fieldcounter     = {}
    # Reihenfolgetreue Liste der vorgesehenen Eingabefelder-Namen
    self.keys             = []
    # Liste der tatsaechlich eingegebenen Eingabefelder-Namen
    self.inputkeys        = []
    # Aufrufmethode
    self.method           = os.environ['REQUEST_METHOD']
    if self.method=='POST':
      if os.environ.has_key('CONTENT_LENGTH'):
        self.contentlength = int(os.environ['CONTENT_LENGTH'])
      else:
        self.contentlength = 0
    else:
      self.contentlength = len(os.environ['QUERY_STRING'])
      # bei GET lesen wir nix von Standardeingabe
      sys.stdin.close()
    self.maxcontentlength = 0

  # Feld hinzufuegen
  def add(self,formfield):
    if self.field.has_key(formfield.name):
      formfield.counter = self.fieldcounter[formfield.name]
      self.field[formfield.name].append(formfield)
      self.fieldcounter[formfield.name] = self.fieldcounter[formfield.name] + 1
    else:
      formfield.counter = 0
      self.field[formfield.name] = [formfield]
      self.keys.append(formfield.name)
      self.fieldcounter[formfield.name] = 1
    self.maxcontentlength = self.maxcontentlength + formfield.maxlength + len(formfield.name) + 2

  # Alle Eingaben in inputform auf Gueltigkeit ueberpruefen.
  # Im Fehlerfall wird eine Ausnahme der Klasse formException generiert
  # Der Parameter encodingdepth gibt an wieviel Stufen des urllib.unquote
  # durchgefhrt werden mssen
  def getparams(self,encodingdepth=1):

    # Ueberpruefen der gesamten Laenge aller Eingaben
    if self.contentlength > self.maxcontentlength:
      raise formContentLengthException(self.contentlength)

    # Eingaben holen
    if self.method=='POST':
      # POST-Methode => Eingaben von stdin lesen
      query_string = ''
      s = sys.stdin.readline()
      while s:
        query_string = query_string + s
        s = sys.stdin.readline()
      sys.stdin.close()
    else:
      # GET-Methode => Eingaben aus der Umgebungsvariable QUERY_STRING lesen
      query_string = os.environ['QUERY_STRING']

    # Parameterliste auseinandernehmen
    inputlist = string.split(query_string,'&')

    # Ueberhaupt Parameter angegeben?
    if not len(inputlist):
      return

    self.contentlength = 0
    # Zaehlerdict. mit Index paramname
    paramnumindex = {}

    # Parameterliste abarbeiten
    for param in inputlist:

      if param:

        # Einzelne Parametername/-daten-Paare auseinandernehmen
        paramname,paramdata = string.split(param,'=',1)

        # Erst hier Unqouting durchfuehren
        for i in range(encodingdepth):
	  paramname = string.strip(urllib.unquote_plus(paramname))
	  paramdata = string.strip(urllib.unquote_plus(paramdata))

        self.contentlength = self.contentlength+1+len(paramname)+len(paramdata)

        # Gesamtlaenge der Daten noch zulaessig?
        if self.contentlength > self.maxcontentlength:
          formContentLengthException(self.contentlength)

        # Unbekannter Parametername angegeben?
	if not (paramname in self.keys):
          raise formParamNameException(paramname)

        if paramnumindex.has_key(paramname):
	  paramnumindex[paramname] = paramnumindex[paramname] + 1
	else:
	  paramnumindex[paramname] = 0

        # Anzahl der Parameter gleichen Namens noch zulaessig?
	if paramnumindex[paramname] >= len(self.field[paramname]):
          raise formParamStructException(paramname,paramnumindex[paramname]+1)

        field = self.field[paramname][paramnumindex[paramname]]

        # kein Leerstring als Parameter?
	if paramdata:
          # Zusaetzlich pruefen
          # Laenge gueltig?
          if len(paramdata) > field.maxlength:
            raise formParamLengthException(field.name,field.text,len(paramdata))
          rm = field.regex.match(paramdata)
          if rm==None or rm.string!=paramdata:
            raise formParamContentException(field.name,field.text,paramdata,field.regex.pattern)
	  # Eingabe ist gueltig und wird in content uebernommen
          field.put(paramdata)
	else:
          field.put('')

        if not paramname in self.inputkeys:
	  self.inputkeys.append(paramname)

    # Alle Parameter mit required-Flag vorhanden?
    for param in self.keys:
      for i in self.field[param]:
        if i.required and not i.content:
          raise formParamNameMissing(i.name,i.text)

    return
