########################################################################
# charset.py Version 0.2.2
# (c) by Michael Stroeder, michael.stroeder@propack-data.de
########################################################################
# Modul zur Umsetzung verschiedener Zeichensaetze
# Anmerkungen:
# - der Quellcode ist z.Zt. nicht plattformunabhaengig, da die
#   ISO-Umlaute direkt als Zeichen angegeben sind.
# - z.Zt. nur fuer deutsche Umlaute, die Tabellen koennen aber sehr
#   leicht erweitert werden
########################################################################

import string

# Diese Definitionen entsprechen denen im Modul string,
# jedoch mit ISO-Umlauten
iso_lowercase='abcdefghijklmnopqrstuvwxyz'
iso_uppercase='ABCDEFGHIJKLMNOPQRSTUVWXYZ'

# trans-Tabellen initialisieren
iso_trans_lower=string.maketrans(iso_uppercase,iso_lowercase)
iso_trans_upper=string.maketrans(iso_lowercase,iso_uppercase)

def lower(s):
  return string.translate(s,iso_trans_lower)

def upper(s):
  return string.translate(s,iso_trans_upper)

# Tabellen fuer die Sonderzeichen, z.Zt. nur deutsche Umlaute
iso =  ('',     '',     '',     '',     '',     '',     ''      )
t61 =  ('\\xE4', '\\xF6', '\\xFC', '\\xC4', '\\xD6', '\\xDC', '\\xDF'  )
html = ('&auml;','&ouml;','&uuml;','&Auml;','&Ouml;','&Uuml;','&szlig;')

########################################################################
# Funktion, welche die angegebene transtable auf den String s anwendet
# 
# Eingabe:
# s		zu modifizierender String
# source	Tupel mit Quellstrings
# target	Tupel mit Zielstrings
# Ausgabe:
# mit transtable modifizierter Eingabestring s
########################################################################

def trans(s,source,target):
  for i in range(len(source)):
    s = string.replace(s,source[i],target[i])
  return s

########################################################################
# Funktion, welche ISO-8859-1 in UTF-8 (Unicode) wandelt
# Eingabe:
# s		String mit ISO-Zeichen
# Ausgabe:
# zu UTF-8 gewandelter Eingabestring
########################################################################

def iso2utf(s):
  new = ''
  for ch in s:
    c=ord(ch)
    if (c & 0x80) == 0:
      new = new+ch
    else:
      new = new+chr(0xC0 | (0x03 & (c >> 6)))+chr(0x80 | (0x3F & c))
  return new

UTF8len= ( 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	   2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 5, 6 )

########################################################################
# Funktion, welche UTF-8 (Unicode) nach ISO-8859-1 wandelt
# nicht in ISO-8859-1 darstellbare Zeichen werden zu '?'
# Eingabe:
# s		String mit Zeichen in UTF-8
# Ausgabe:
# zu ISO gewandelter Eingabestring
########################################################################

def utf2iso(s):

  new = ''
  ind = 0
  slen = len(s)

  while ind < slen:
    c=ord(s[ind])
    ind = ind+1 
    clen = UTF8len [(c >> 2) & 0x3F]
    if   clen==6: u = c & 0x01
    elif clen==5: u = c & 0x03
    elif clen==4: u = c & 0x07
    elif clen==3: u = c & 0x0F
    elif clen==2: u = c & 0x1F
    elif clen==1: u = c & 0x7F
    elif clen==0: clen = 5; u = c & 0x3F
    clen = clen-1
    while clen and ind < slen:
      c=ord(s[ind])
      ind = ind+1 
      if (c & 0xC0) == 0x80:
        u = (u << 6) | (c & 0x3F)
      else:
        ind = ind-1
        break
      clen = clen-1
    
    if (u <= 0xFF):
      new = new+chr(u)
    else:
      new = new+'?'

  return new

########################################################################
# Funktion, welche UTF-8 (Unicode) nach HTML 4 konforme
# Unicode-Darstellung wandelt
# Eingabe:
# s		String mit Zeichen in UTF-8
# Ausgabe:
# zu HTML 4.0 gewandelter Eingabestring
########################################################################

def utf2html4(s,raw=0):

  new = ''
  ind = 0
  slen = len(s)

  while ind < slen:
    c=ord(s[ind])
    ind = ind+1 
    clen = UTF8len [(c >> 2) & 0x3F]
    if   clen==6: u = c & 0x01
    elif clen==5: u = c & 0x03
    elif clen==4: u = c & 0x07
    elif clen==3: u = c & 0x0F
    elif clen==2: u = c & 0x1F
    elif clen==1: u = c & 0x7F
    elif clen==0: clen = 5; u = c & 0x3F
    clen = clen-1
    while clen and ind < slen:
      c=ord(s[ind])
      ind = ind+1 
      if (c & 0xC0) == 0x80:
        u = (u << 6) | (c & 0x3F)
      else:
        ind = ind-1
        break
      clen = clen-1
    
    if u<128 and not raw:
      new = new + chr(u)
    else:
      new = new + '&#%d;' % u

  return new

########################################################################
# Funktion, welche ISO-Zeichen nach HTML 4 konforme
# Unicode-Darstellung wandelt
# Eingabe:
# s		String mit Zeichen in ISO-8859-1
# Ausgabe:
# zu HTML 4.0 gewandelter Eingabestring
########################################################################

def iso2html4(s,raw=0):

  new = ''
  for ch in s:
    c=ord(ch)
    if (c & 0x80)==0 and not raw:
      new = new + ch
    else:
      new = new + '&#%d;' % (c)
  return new
